function yA = GenerateGrahamData(Param)
% This routine generates dynamic or static spatial panel data with weight
% matrices generated according to Graham(2016)
% The routine also prepares the data for the estimation routines, most
% importantly by generating a set of linear and quadratic instruments


n          = Param.n;
T          = Param.T;
amu        = Param.amu;
mu_gam     = Param.mu_gam;
kappa      = Param.kappa;
azeta      = Param.azeta;
a0         = Param.a0;
beta1      = Param.beta1;  % parameter for y_t-1
beta2      = Param.beta2;  % parameter for M_t*y_t-1
beta3      = Param.beta3;  % parameter on exogenous Z_t
beta4      = Param.beta4;  % parameter on M_t*Z_t
delta_1    = Param.delta_1;
delta_2    = Param.delta_2;
lambda     = Param.lambda;
facalph    = Param.factoralph;
facsig     = Param.factorsig;
zetadep    = Param.zetadep;
zetasig    = Param.zetasig;
gamma      = Param.gamma;   %0 or 1 depending on whether M is endogenous or not
sigmav     = Param.sigmav;
factors    = Param.factors;
projectA   = Param.projectA;
param      = Param;
staticM    = Param.staticM;

zeta       = rand(n,1)*2;
shz        = (1:n)';
zeta       = zeta + shz;

zeta       = genZeta(zeta,n,T,zetadep,zetasig);

if staticM
    ups        = InverseLogistic(rand(n^2,1))*ones(1,T);  %if M is static draw same shock for all periods
else
    ups        = InverseLogistic(rand(n^2,T));
end
ups        = SymmetrizeUPS(ups);

v          = sigmav * randn(n,T);  % generate nxT vector of disturbances
Z          = randn(n,T);  % generate exogenous covariates
if (beta2~=0)
    mui        = randn(n,1); %generate endogenous mu
else
    mui        = (mu_gam * Z *ones(T,1)/sqrt(T) + randn(n,1))/sqrt(mu_gam^2 + 1); %generate endogenous mu
end
nui        = randn(n,1);
f          = ones(T,1);
A          = zeros(n,n);
AmF        = zeros(4*n^2,T);  % store A matrices
AmFstar    = zeros(4*n^2,T);  % store A matrices endogenous M case
VmF        = zeros(4^2,T);    % store GMM quad moment weight matrix
VmFstar    = zeros(4^2,T);    % store GMM quad moment weight matrix endogenous M case
M          = zeros(n^2,T);    % store M (observed data and possible input to instruments
Mstar      = zeros(n^2,T);    % store approximate M to handle endogenous case
D          = zeros(n^2,T);
M1         = zeros(n,n);
d1         = zeros(n,n);
y          = zeros(n,T);
H          = zeros(6*n,T);     % Instruments
H1         = zeros(8*n,T);     % Instruments
H2         = zeros(3*n,T);     % Instruments

HA         = zeros(n,T);       % Instruments
Hstar	   = zeros(6*n,T);     % Instruments based on Mstar
H1star	   = zeros(8*n,T);     % Instruments based on Mstar
H2star	   = zeros(3*n,T);     % Instruments based on Mstar

Hdyn       = zeros(5*n,T);     % Instruments for dynamic model
HAdyn      = zeros(n,T);       % Instruments for dynamic model
Hstardyn   = zeros(5*n,T);     % Instruments for dynamic model using Mstar

Hdyn1      = zeros(7*n,T);     % Instruments for dynamic model
HAdyn1     = zeros(n,T);       % Instruments for dynamic model
Hstardyn1  = zeros(7*n,T);     % Instruments for dynamic model using Mstar

Hdyn2      = zeros(3*n,T);     % Instruments for dynamic model
HAdyn2     = zeros(n,T);       % Instruments for dynamic model
Hstardyn2  = zeros(3*n,T);     % Instruments for dynamic model using Mstar


MZplus     = zeros(n,T);


if factors
    f(1,1)     = randn(1,1); %initialize factor process
end

% Compute initial condition for M1
ups1       = reshape(ups(:,1),n,n);
for i=1:n
    zetai  = zeta(i,1);
    mui_i  = mui(i);
    nui_i  = nui(i);
    M1i    = zeros(1,n);
    d1i    = zeros(1,n);
    
    for j=1:n
        A(i,j)    = nui_i+nui(j)+ amu*EuclidNorm(mui_i-mui(j));
        ups1(i,j) = gamma*(v(i,1)+v(j,1))/2 + ups1(i,j);
        D1ij      = a0 +azeta*EuclidNorm((zeta(j,1) -zetai)')+ A(i,j) + ups1(i,j);
        sij       = (EuclidNorm((zeta(j,1) -zetai)') < kappa/2);
        M1i(j)    = (D1ij>0)*sij;
        d1i(j)    = (D1ij>0)*sij;
    end
    M1(i,:)  = M1i;
    M1(i,i)  = 0;
    M1s      = M1(i,:)*ones(n,1);
    d1(i,:)  = M1(i,:); %store adjacency matrix before normalization
    if M1s==0
        M1(i,:)  = zeros(1,n);
    else
        M1(i,:)  = M1(i,:)./(ones(1,n)*M1s);
    end
end

M(:,1)       = vec(M1);
M1star       = ApproximateM(zeta(:,1),param,[]); % approximate M matrix for the endogenous case
Mstar(:,1)   = vec(M1star);
D(:,1)       = vec(d1);
R1           = d1*d1';


if beta1~=0
    if rank((eye(n)-lambda*M1)-(eye(n)*beta1+M1*beta2))<n
        u(:,1)       = v(:,1) + mui*f(1,1);
    else
        u(:,1)      = v(:,1) + inv((eye(n)-lambda*M1)-(eye(n)*beta1+M1*beta2))*mui*f(1,1);
    end
    y(:,1)       = u(:,1);   %initialize dynamic model
else
    u(:,1)       = v(:,1) + mui*f(1,1);
    y(:,1)       = inv(eye(n)-lambda*M1)*(Z(:,1)*beta3 + M1*Z(:,1)*beta4 + u(:,1));
end

% recursively generate Mt and yt for t=2 to T
for t = 2:T
    if factors
        f(t,1)      = facalph * f(t-1,1) + facsig * randn(1,1);      % factor is a unit root with gaussian innovations
    else
        % do nothing since initialize f = 1 model is
        % automatically a fixed effects model
    end
    [M1,R1,d1]     = GenMt(A,R1,d1,ups,v,zeta(:,t),gamma,kappa,a0,azeta,delta_1,delta_2,n,t);
    
    [Am,Vm]        = ConstructA_M(M1,reshape(M(:,t-1),n,n),projectA);  % compute A matrices for quadratic moments, optimal weights
    AmF(:,t-1)     = Am;                                      % store vectorized A matrix
    VmF(:,t-1)     = vec(Vm);                                 % store vecotrized weight matrix
    M1star         = ApproximateM(zeta(:,t),param,reshape(Mstar(:,t-1),n,n));             % approximate M matrix for the endogenous case
    
    [Am,Vm]        = ConstructA_M(M1star,reshape(Mstar(:,t-1),n,n),projectA); % construct A matrix using approximations
    AmFstar(:,t-1) = Am;                                      % store approximate A matrix for further use GMM construction
    VmFstar(:,t-1) = vec(Vm);                                 % store V matrix using exogenous approximation
    
    M(:,t)       = vec(M1);
    Mstar(:,t)   = vec(M1star);
    D(:,t)       = vec(d1);
    u(:,t)   = v(:,t) + mui*f(t,1);                      % static model model
    
    y(:,t)       = inv(eye(n) - lambda*M1)*((eye(n)*beta1+reshape(M1,n,n)*beta2)*y(:,t-1) + Z(:,t)*beta3 + reshape(M1,n,n)*Z(:,t)*beta4 + u(:,t));
    
    Mt           = reshape(M(:,t),n,n);
    Mt1          = reshape(M(:,t-1),n,n);
    Mts          = reshape(Mstar(:,t),n,n);
    Mts1         = reshape(Mstar(:,t-1),n,n);
    
    if t>2
        Mt2          = reshape(M(:,t-2),n,n);
        Mts2         = reshape(Mstar(:,t-2),n,n);
    end
    
    if beta1 ~= 0
        if t>2
            Hdyn(:,t)      = [y(:,t-2);Mt2*y(:,t-2);Mt2*Mt2*y(:,t-2);Mt1*y(:,t-2);Mt1*Mt1*y(:,t-2)];
            HAdyn(:,t)     =  y(:,t-2);
            Hstardyn(:,t)  = [y(:,t-2);Mts2*y(:,t-2);Mts2*Mts2*y(:,t-2);Mts1*y(:,t-2);Mts1*Mts1*y(:,t-2)];
            Hdyn1(:,t)     = [y(:,t-2);Mt2*y(:,t-2);Mt2*Mt2*y(:,t-2);Mt1*y(:,t-2);Mt1*Mt1*y(:,t-2);Mt2*Mt2*Mt2*y(:,t-2);Mt1*Mt1*Mt1*y(:,t-2)];
            HAdyn1(:,t)    =  y(:,t-2);
            Hstardyn1(:,t) = [y(:,t-2);Mts2*y(:,t-2);Mts2*Mts2*y(:,t-2);Mts1*y(:,t-2);Mts1*Mts1*y(:,t-2);Mts2*Mts2*Mts2*y(:,t-2);Mts1*Mts1*Mts1*y(:,t-2)];
            Hdyn2(:,t)     = [y(:,t-2);Mt2*y(:,t-2);Mt2*Mt2*y(:,t-2)];
            HAdyn2(:,t)    =  y(:,t-2);
            Hstardyn2(:,t) = [y(:,t-2);Mts2*y(:,t-2);Mts2*Mts2*y(:,t-2)];
        end
    else
        HA(:,t)      =  Z(:,t);
        H(:,t)       = [Z(:,t)-Z(:,t-1);Z(:,t-1);Mt*Z(:,t)-Mt1*Z(:,t-1);Mt1*Z(:,t-1);Mt*Mt*Z(:,t);Mt1*Mt1*Z(:,t-1)];
        Hstar(:,t)   = [Z(:,t)-Z(:,t-1);Z(:,t-1);Mts*Z(:,t)-Mts1*Z(:,t-1);Mts1*Z(:,t-1);Mts*Mts*Z(:,t);Mts1*Mts1*Z(:,t-1)];
        H1(:,t)      = [Z(:,t)-Z(:,t-1);Mt*Z(:,t)-Mt1*Z(:,t-1);Z(:,t-1);Mt1*Z(:,t-1);Mt*Mt*Z(:,t);Mt1*Mt1*Z(:,t-1);Mt*Mt*Mt*Z(:,t);Mt1*Mt1*Mt1*Z(:,t-1)];
        H1star(:,t)  = [Z(:,t)-Z(:,t-1);Mts*Z(:,t)-Mts1*Z(:,t-1);Z(:,t-1);Mts1*Z(:,t-1);Mts*Mts*Z(:,t);Mts1*Mts1*Z(:,t-1);...
                        Mts*Mts*Mts*Z(:,t);Mts1*Mts1*Mts1*Z(:,t-1)];
        H2(:,t)      = [Z(:,t)-Z(:,t-1);Mt*Z(:,t)-Mt1*Z(:,t-1);Mt*Mt*Z(:,t)-Mt1*Mt1*Z(:,t-1)];
        H2star(:,t)  = [Z(:,t)-Z(:,t-1);Mts*Z(:,t)-Mts1*Z(:,t-1);Mts*Mts*Z(:,t)-Mts1*Mts1*Z(:,t-1)];
    end
end

if beta1~=0
    Hdyn           = cleanH(Hdyn,n);
    Hstardyn       = cleanH(Hstardyn,n);
    Hdyn           = Hdyn(:,3:end);
    HAdyn          = HAdyn(:,3:end);
    Hstardyn       = Hstardyn(:,3:end);
    
    Hdyn1          = cleanH(Hdyn1,n);
    Hstardyn1      = cleanH(Hstardyn1,n);
    Hdyn1          = Hdyn1(:,3:end);
    Hstardyn1      = Hstardyn1(:,3:end);
    
    Hdyn2          = cleanH(Hdyn2,n);
    Hstardyn2      = cleanH(Hstardyn2,n);
    Hdyn2          = Hdyn2(:,3:end);
    HAdyn2         = HAdyn2(:,3:end);
    Hstardyn2      = Hstardyn2(:,3:end);
    
    
else
    H              = cleanH(H,n);
    Hstar          = cleanH(Hstar,n);
    H1             = cleanH(H1,n);
    H1star         = cleanH(H1star,n);
    H2             = cleanH(H2,n);
    H2star         = cleanH(H2star,n);
    
    H              = H(:,2:end);
    HA             = HA(:,2:end);
    Hstar          = Hstar(:,2:end);
    H1             = H1(:,2:end);
    H2             = H2(:,2:end);
    H1star         = H1star(:,2:end);
    H2star         = H2star(:,2:end);
end

sigma         = ones(T,1);
dynamic       = (beta1 ~= 0);
[yplus,yMplus,yMLplus,Wplus,W1plus,W2plus,Zplus,zplus]= ForwardDiff(y,Z,M,sigma,dynamic);


yA.yplusDat   = yplus;
yA.yMplusDat  = yMplus;
yA.yMLplusDat = yMLplus;
yA.yDat       = y;
yA.zetaDat    = zeta;
yA.ZDat       = Z;
yA.AmFDat     = AmF;
yA.AmFstarDat = AmFstar;
yA.MDat       = M;
yA.MstarDat   = Mstar;
yA.VmFDat     = VmF;
yA.VmFstarDat = VmFstar;
yA.DDat       = D;
yA.WplusDat   = Wplus;
yA.W1plusDat  = W1plus;
yA.W2plusDat  = W2plus;
yA.ZplusDat   = Zplus;
yA.zplusDat   = zplus;
yA.MZplusDat  = MZplus;
yA.HDat       = H;
yA.H1Dat      = H1;
yA.H2Dat      = H2;
yA.HADat      = HA;

yA.HstarDat   = Hstar;

yA.H1starDat  = H1star;
yA.H2starDat  = H2star;

yA.HdynDat    = Hdyn;
yA.HAdynDat   = HAdyn;
yA.HstardynDat= Hstardyn;

yA.Hdyn1Dat    = Hdyn1;
yA.HAdyn1Dat   = HAdyn1;
yA.Hstardyn1Dat= Hstardyn1;

yA.Hdyn2Dat    = Hdyn2;
yA.HAdyn2Dat   = HAdyn2;
yA.Hstardyn2Dat= Hstardyn2;

yA.dynamic    = dynamic;

end